import api = require('@aws-cdk/aws-autoscaling-api');
import iam = require('@aws-cdk/aws-iam');
import cdk = require('@aws-cdk/cdk');
import { IAutoScalingGroup } from './auto-scaling-group';
/**
 * Basic properties for a lifecycle hook
 */
export interface BasicLifecycleHookProps {
    /**
     * Name of the lifecycle hook
     *
     * @default Automatically generated name
     */
    readonly lifecycleHookName?: string;
    /**
     * The action the Auto Scaling group takes when the lifecycle hook timeout elapses or if an unexpected failure occurs.
     *
     * @default Continue
     */
    readonly defaultResult?: DefaultResult;
    /**
     * Maximum time between calls to RecordLifecycleActionHeartbeat for the hook
     *
     * If the lifecycle hook times out, perform the action in DefaultResult.
     */
    readonly heartbeatTimeoutSec?: number;
    /**
     * The state of the Amazon EC2 instance to which you want to attach the lifecycle hook.
     */
    readonly lifecycleTransition: LifecycleTransition;
    /**
     * Additional data to pass to the lifecycle hook target
     *
     * @default No metadata
     */
    readonly notificationMetadata?: string;
    /**
     * The target of the lifecycle hook
     */
    readonly notificationTarget: api.ILifecycleHookTarget;
    /**
     * The role that allows publishing to the notification target
     *
     * @default A role is automatically created
     */
    readonly role?: iam.IRole;
}
/**
 * Properties for a Lifecycle hook
 */
export interface LifecycleHookProps extends BasicLifecycleHookProps {
    /**
     * The AutoScalingGroup to add the lifecycle hook to
     */
    readonly autoScalingGroup: IAutoScalingGroup;
}
export declare class LifecycleHook extends cdk.Construct implements api.ILifecycleHook {
    /**
     * The role that allows the ASG to publish to the notification target
     */
    readonly role: iam.IRole;
    /**
     * The name of this lifecycle hook
     */
    readonly lifecycleHookName: string;
    constructor(scope: cdk.Construct, id: string, props: LifecycleHookProps);
}
export declare enum DefaultResult {
    Continue = "CONTINUE",
    Abandon = "ABANDON"
}
/**
 * What instance transition to attach the hook to
 */
export declare enum LifecycleTransition {
    /**
     * Execute the hook when an instance is about to be added
     */
    InstanceLaunching = "autoscaling:EC2_INSTANCE_LAUNCHING",
    /**
     * Execute the hook when an instance is about to be terminated
     */
    InstanceTerminating = "autoscaling:EC2_INSTANCE_TERMINATING"
}
